/**
 * Copyright (C) 2011 Linutop SAS <http://www.linutop.com/>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#define _BSD_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#define DEFAULT_PWDFILE "/cdrom/tickets"

char *pwdfile = NULL;
int nb = 0;

void
usage ()
{
    fprintf (stderr, "Usage:\n");
    fprintf (stderr, "  kiosk-generat-tickets [-a] [-f FILENAME] NB\n");
    fprintf (stderr, "\n");
    fprintf (stderr, "  -a will append the generated tickets to the existing list\n");
    fprintf (stderr, "  -f will write in FILENAME (default: %s)\n", DEFAULT_PWDFILE);
    fprintf (stderr, "\n");
}

int
generate (int nb, int append)
{
    char cmd[254];
    char add[3];
    int ret;

    strcpy (add, append ? ">>" : ">");
    snprintf (cmd, 253, "pwgen -1 -N %d | tr 'a-z' 'A-Z' | sort -u %s %s",
              nb, add, pwdfile);
    chmod (pwdfile, 0600);
    ret = system (cmd);

    return ret;
}

int
main (int argc, char **argv)
{
    int append = 0;
    int tmp;
    int i;
    int ret;

    if (setuid (0) != 0)
    {
        fprintf (stderr, "Need to be root.\n");
        return (1);
    }

    for (i=1; i<argc; i++)
    {
        if (strcmp (argv[i], "-a") == 0)
            append = 1;
        else if ((tmp = atoi (argv[i])))
            nb = tmp;
        else if (strcmp (argv[i], "-f") == 0)
        {
            if (i == (argc-1))
            {
                usage ();
                return 1;
            }
            pwdfile = strdup (argv[i+1]);
            i++;
        }
        else
        {
            usage ();
            return 1;
        }
    }
    if (!nb)
    {
        usage ();
        return 1;
    }
    if (!pwdfile)
        pwdfile = strdup (DEFAULT_PWDFILE);

    ret = generate (nb, append);

    free (pwdfile);

    return ret;
}

